@@ -25,9 +25,7 @@ from .iterators import BIterator
 try:
     from xplane_airports import AptDat
 except ImportError as e:
-    raise ImportError(
-        "Could not import xplane_airports. Install https://github.com/X-Plane/xplane_airports."
-    ) from e
+    raise ImportError("Could not import xplane_airports. Install https://github.com/X-Plane/xplane_airports.") from e
 
 
 logging.basicConfig(
@@ -36,7 +34,7 @@ logging.basicConfig(
     datefmt="[%X]",
     handlers=[RichHandler(show_path=False, omit_repeated_times=False)],
 )
-logger = logging.getLogger(__name__)
+logger = logging.getLogger("xplane_apt_convert")
 logger.setLevel(logging.INFO)
 
 
@@ -169,9 +167,7 @@ class ParsedAirport:
             elif row_code == AptDat.RowCode.BOUNDARY:
                 logger.debug("Parsing boundary row.")
 
-                boundary = Boundary.from_row_iterator(
-                    row, row_iterator, bezier_resolution
-                )
+                boundary = Boundary.from_row_iterator(row, row_iterator, bezier_resolution)
 
                 if boundary is not None:
                     self.boundary = boundary
@@ -213,9 +209,7 @@ class ParsedAirport:
             elif row_code == AptDat.RowCode.TAXIWAY:
                 logger.debug("Parsing pavement row.")
 
-                pavement = Pavement.from_row_iterator(
-                    row, row_iterator, bezier_resolution
-                )
+                pavement = Pavement.from_row_iterator(row, row_iterator, bezier_resolution)
 
                 if pavement is not None:
                     self.pavements.append(pavement)
@@ -223,9 +217,7 @@ class ParsedAirport:
             elif row_code == AptDat.RowCode.FREE_CHAIN:
                 logger.debug("Parsing linear feature row.")
 
-                for line in LinearFeature.from_row_iterator(
-                    row, row_iterator, bezier_resolution
-                ):
+                for line in LinearFeature.from_row_iterator(row, row_iterator, bezier_resolution):
                     if line is not None:
                         self.linear_features.append(line)
 
@@ -252,15 +244,9 @@ class ParsedAirport:
                 Default is all.
         """
         if driver is not None and driver not in SUPPORTED_DRIVERS:
-            raise ValueError(
-                f"Invalid driver '{driver}'. Supported drivers: {list(SUPPORTED_DRIVERS.keys())}"
-            )
+            raise ValueError(f"Invalid driver '{driver}'. Supported drivers: {list(SUPPORTED_DRIVERS.keys())}")
 
-        if (
-            isinstance(output_path, str)
-            and len(output_path) > 0
-            and output_path[-1] == "/"
-        ):
+        if isinstance(output_path, str) and len(output_path) > 0 and output_path[-1] == "/":
             raise ValueError("output_path cannot be a directory.")
 
         if not isinstance(output_path, Path):
@@ -291,9 +277,7 @@ class ParsedAirport:
 
         for part_name in features:
             if part_name not in VALID_FEATURES:
-                raise ValueError(
-                    f"{part_name} is not a valid include. Valid options are: {VALID_FEATURES}"
-                )
+                raise ValueError(f"{part_name} is not a valid include. Valid options are: {VALID_FEATURES}")
 
             part = getattr(self, part_name)
 
@@ -347,8 +331,6 @@ class ParsedAirport:
 
                 if _BASE_CRS != crs:
                     for record in records:
-                        record["geometry"] = transform_geom(
-                            _BASE_CRS, crs, record["geometry"]
-                        )
+                        record["geometry"] = transform_geom(_BASE_CRS, crs, record["geometry"])
 
                 output.writerecords(records)
