@@ -1,7 +1,8 @@
 from abc import ABC, abstractmethod
 from collections import OrderedDict
 from dataclasses import dataclass
-from enum import Enum
+from enum import Enum, EnumMeta
+import logging
 
 from .geometry import get_paths
 from .iterators import BIterator
@@ -9,12 +10,115 @@ from .iterators import BIterator
 try:
     from xplane_airports import AptDat
 except ImportError as e:
-    raise ImportError(
-        "Could not import xplane_airports. See https://github.com/X-Plane/xplane_airports."
-    ) from e
+    raise ImportError("Could not import xplane_airports. Install https://github.com/X-Plane/xplane_airports.") from e
 
 
-class SurfaceType(Enum):
+logger = logging.getLogger("xplane_apt_convert")
+logged_unknowns = set()  # TODO: this should be reset with each different airport parsing
+
+
+class FallbackEnumMeta(EnumMeta):
+    class Fallback:
+        def __init__(self, name):
+            self.name = name
+
+    def __call__(cls, value, names=None, *args, **kwargs):
+        try:
+            return EnumMeta.__call__(cls, value, names=None, *args, **kwargs)
+        except ValueError:
+            if names is not None:
+                # using functional API attempting to create a new Enum type
+                raise
+
+            if value is None:
+                return FallbackEnumMeta.Fallback(None)
+            else:
+                if (cls, value) not in logged_unknowns:
+                    logger.warning(f"Unknown value {value} found for {cls.__name__}.")
+                    # do not log same warning many times
+                    logged_unknowns.add((cls, value))
+
+                return FallbackEnumMeta.Fallback(f"UNKNOWN_{value}")
+
+
+class FallbackEnum(Enum, metaclass=FallbackEnumMeta):
+    """Custom subclass of Enum that supports handling of arbitrary unknown values."""
+
+
+class LineType(FallbackEnum):
+    NONE = 0
+    SOLID_YELLOW = 1
+    BROKEN_YELLOW = 2
+    DOUBLE_SOLID_YELLOW = 3
+    RUNWAY_HOLD = 4  # DOUBLE_BROKEN_YELLOW_WITH_PARALLEL_DOUBLE_SOLID_YELLOW = 4
+    OTHER_HOLD = 5  # BROKEN_YELLOW_WITH_PARALLEL_SOLID_YELLOW = 5
+    ILS_HOLD = 6  # YELLOW_CROSSHATCHED = 6
+    ILS_CRITICAL_CENTERLINE = 7
+    SEPARATED_BROKEN_YELLOW = 8
+    SEPARATED_DOUBLE_BROKEN_YELLOW = 9
+    WIDE_SOLID_YELLOW = 10
+    WIDE_ILS_CRITICAL_CENTERLINE = 11
+    WIDE_RUNWAY_HOLD = 12
+    WIDE_OTHER_HOLD = 13
+    WIDE_ILS_HOLD = 14
+
+    SOLID_YELLOW_WITH_BLACK_BORDER = 51
+    BROKEN_YELLOW_WITH_BLACK_BORDER = 52
+    DOUBLE_SOLID_YELLOW_WITH_BLACK_BORDER = 53
+    RUNWAY_HOLD_WITH_BLACK_BORDER = 54
+    OTHER_HOLD_WITH_BLACK_BORDER = 55
+    ILS_HOLD_WITH_BLACK_BORDER = 56
+    ILS_CRITICAL_CENTERLINE_WITH_BLACK_BORDER = 57
+    SEPARATED_BROKEN_YELLOW_WITH_BLACK_BORDER = 58
+    SEPARATED_DOUBLE_BROKEN_YELLOW_WITH_BLACK_BORDER = 59
+    WIDE_SOLID_YELLOW_WITH_BLACK_BORDER = 60
+    WIDE_ILS_CRITICAL_CENTERLINE_WITH_BLACK_BORDER = 61
+    WIDE_RUNWAY_HOLD_WITH_BLACK_BORDER = 62
+    WIDE_OTHER_HOLD_WITH_BLACK_BORDER = 63
+    WIDE_ILS_HOLD_WITH_BLACK_BORDER = 64
+
+    VERY_WIDE_YELLOW = 19
+
+    SOLID_WHITE = 20
+    CHEQUERED_WHITE = 21
+    BROKEN_WHITE = 22
+    SHORT_BROKEN_WHITE = 23
+    WIDE_SOLID_WHITE = 24
+    WIDE_BROKEN_WHITE = 25
+    SOLID_RED = 30
+    BROKEN_RED = 31
+    WIDE_SOLID_RED = 32
+    SOLID_ORANGE = 40
+    SOLID_BLUE = 41
+    SOLID_GREEN = 42
+
+    SOLID_WHITE_WITH_BLACK_BORDER = 70
+    CHEQUERED_WHITE_WITH_BLACK_BORDER = 71
+    BROKEN_WHITE_WITH_BLACK_BORDER = 72
+    SHORT_BROKEN_WHITE_WITH_BLACK_BORDER = 73
+    WIDE_SOLID_WHITE_WITH_BLACK_BORDER = 74
+    WIDE_BROKEN_WHITE_WITH_BLACK_BORDER = 75
+    SOLID_RED_WITH_BLACK_BORDER = 80
+    BROKEN_RED_WITH_BLACK_BORDER = 81
+    WIDE_SOLID_RED_WITH_BLACK_BORDER = 82
+    SOLID_ORANGE_WITH_BLACK_BORDER = 90
+    SOLID_BLUE_WITH_BLACK_BORDER = 91
+    SOLID_GREEN_WITH_BLACK_BORDER = 92
+
+
+class LineLightingType(FallbackEnum):
+    NONE = 0
+    GREEN_BIDIRECTIONAL_LIGHTS = 101
+    BLUE_OMNIDIRECTIONAL_LIGHTS = 102
+    AMBER_UNIDIRECTIONAL_LIGHTS = 103
+    AMBER_UNIDIRECTIONAL_PULSATING_LIGHTS = 104
+    ALTERNATING_AMBER_GREEN_BIDIRECTIONAL_LIGHTS = 105
+    RED_OMNIDIRECTIONAL_LIGHTS = 106
+    GREEN_UNIDIRECTIONAL_LIGHTS = 107
+    ALTERNATING_AMBER_GREEN_UNIDIRECTIONAL_LIGHTS = 108
+
+
+class SurfaceType(FallbackEnum):
     ASPHALT = 1  # Asphalt
     CONCRETE = 2  # Concrete
     TURF_OR_GRASS = 3  # Turf or grass
@@ -26,22 +130,44 @@ class SurfaceType(Enum):
     TRANSPARENT = 15  # Transparent
 
 
-class ShoulderSurfaceType(Enum):
+class ShoulderSurfaceType(FallbackEnum):
     NONE = 0  # No shoulder
     ASPHALT = 1  # Asphalt shoulder
     CONCRETE = 2  # Concrete shoulder
 
 
-class RunwayMarking(Enum):
+class RunwayMarking(FallbackEnum):
     NONE = 0  # No runway markings
     VISUAL = 1  # Visual markings
-    NON_PRECISION = 3  # Non-precision approach markings
-    PRECISION = 4  # Precision approach markings
-    UK_NON_PRECISION = 5  # UK-style non-precision approach markings
+    NON_PRECISION = 2  # Non-precision approach markings
+    PRECISION = 3  # Precision approach markings
+    UK_NON_PRECISION = 4  # UK-style non-precision approach markings
     UK_PRECISION = 5  # UK-style precision approach markings
 
 
-class SignSize(Enum):
+class ApproachLighting(FallbackEnum):
+    NONE = 0  # No approach lighting
+    ALSF_I = 1  # ALSF-I. Approach Lighting System with Sequenced Flashing Lights
+    ALSF_II = 2  # ALSF-II. Approach Lighting System with Sequenced Flashing Lights
+    CALVERT = 3  # Calvert
+    CALVERT_II = 4  # Calvert ILS Cat II and Cat II
+    SSALR = 5  # SSALR. Simplified Short Approach Lighting System with Runway Alignment Indicator Lights
+    SSALF = 6  # SSALF. Simplified Short Approach Lighting System with Sequenced Flashing Lights
+    SALS = 7  # SALS. Short Approach Light System
+    MALSR = 8  # MALSR. Medium-intensity Approach Light System with Runway Alignment Indicator Lights
+    MALSF = 9  # MALSF. Medium-intensity Approach Light System with Sequenced Flashing Lights
+    MALS = 10  # MALS. Medium-intensity Approach Light System
+    ODALS = 11  # ODALS. Omni-Directional Approach Light System
+    RAIL = 12  # RAIL. Runway Alignment Indicator Lights
+
+
+class RunwayEndIdentifierLights(FallbackEnum):
+    NONE = 0  # No REIL
+    OMNIDIRECTIONAL_REIL = 1  # Omni-directional REIL
+    UNIDIRECTIONAL_REIL = 2  # Unidirectional REIL
+
+
+class SignSize(FallbackEnum):
     SMALL = 1  # Small taxiway sign
     MEDIUM = 2  # Medium taxiway sign
     LARGE = 3  # Large taxiway sign
@@ -87,9 +213,7 @@ class Boundary(AptFeature):
         )
 
         coordinates_list = [c for c in coordinates_list if len(c) > 2]
-        properties_list = [
-            p for c, p in zip(coordinates_list, properties_list) if len(c) > 2
-        ]
+        properties_list = [p for c, p in zip(coordinates_list, properties_list) if len(c) > 2]
 
         return Boundary(
             name=" ".join(tokens[1:]),
@@ -137,9 +261,7 @@ class Pavement(AptFeature):
         )
 
         coordinates_list = [c for c in coordinates_list if len(c) > 2]
-        properties_list = [
-            p for c, p in zip(coordinates_list, properties_list) if len(c) > 2
-        ]
+        properties_list = [p for c, p in zip(coordinates_list, properties_list) if len(c) > 2]
 
         return Pavement(
             surface_type=SurfaceType(int(tokens[1])),
@@ -208,8 +330,8 @@ class LinearFeature(AptFeature):
         return [
             LinearFeature(
                 name=" ".join(tokens[1:]),
-                painted_line_type=properties.get("painted_line_type"),
-                lighting_line_type=properties.get("lighting_line_type"),
+                painted_line_type=LineType(properties.get("painted_line_type")),
+                lighting_line_type=LineLightingType(properties.get("lighting_line_type")),
                 coordinates=coordinates,
             )
             for coordinates, properties in zip(coordinates_list, properties_list)
@@ -223,8 +345,8 @@ class LinearFeature(AptFeature):
             "properties": OrderedDict(
                 [
                     ("name", "str"),
-                    ("painted_line_type", "int"),
-                    ("lighting_line_type", "int"),
+                    ("painted_line_type", "str"),
+                    ("lighting_line_type", "str"),
                 ]
             ),
         }
@@ -237,8 +359,8 @@ class LinearFeature(AptFeature):
             },
             "properties": {
                 "name": self.name,
-                "painted_line_type": self.painted_line_type,
-                "lighting_line_type": self.lighting_line_type,
+                "painted_line_type": self.painted_line_type.name,
+                "lighting_line_type": self.lighting_line_type.name,
             },
         }
 
@@ -251,8 +373,8 @@ class RunwayEnd:
     dthr_length: float  # Displaced Threshold length in meters
     overrun_length: float  # in meters
     marking: RunwayMarking
-    lighting: int
-    tdz_lighting: int  # Touchdown Zone lighting
+    lighting: ApproachLighting
+    tdz_lighting: bool  # Touchdown Zone lighting
     reil: int  # Runway End Identifier Lights
 
     @staticmethod
@@ -263,10 +385,10 @@ class RunwayEnd:
             longitude=float(tokens[2]),
             dthr_length=float(tokens[3]),
             overrun_length=float(tokens[4]),
-            marking=None,  # TODO
-            lighting=None,  # TODO
-            tdz_lighting=None,  # TODO
-            reil=None,  # TODO
+            marking=RunwayMarking(int(tokens[5])),
+            lighting=ApproachLighting(int(tokens[6])),
+            tdz_lighting=bool(int(tokens[7])),
+            reil=RunwayEndIdentifierLights(int(tokens[8])),
         )
 
 
@@ -311,6 +433,20 @@ class Runway(AptFeature):
                     ("centerline_lights", "int"),
                     ("edge_lights", "int"),
                     ("auto_distance_remaining_signs", "bool"),
+                    ("name_1", "str"),
+                    ("name_2", "str"),
+                    ("dthr_length_1", "float"),
+                    ("dthr_length_2", "float"),
+                    ("overrun_length_1", "float"),
+                    ("overrun_length_2", "float"),
+                    ("marking_1", "str"),
+                    ("marking_2", "str"),
+                    ("lighting_1", "str"),
+                    ("lighting_2", "str"),
+                    ("tdz_lighting_1", "bool"),
+                    ("tdz_lighting_2", "bool"),
+                    ("reil_1", "str"),
+                    ("reil_2", "str"),
                 ]
             ),
         }
@@ -332,6 +468,20 @@ class Runway(AptFeature):
                 "centerline_lights": self.centerline_lights,
                 "edge_lights": self.edge_lights,
                 "auto_distance_remaining_signs": self.auto_distance_remaining_signs,
+                "name_1": self.ends[0].name,
+                "name_2": self.ends[1].name,
+                "dthr_length_1": self.ends[0].dthr_length,
+                "dthr_length_2": self.ends[1].dthr_length,
+                "overrun_length_1": self.ends[0].overrun_length,
+                "overrun_length_2": self.ends[1].overrun_length,
+                "marking_1": self.ends[0].marking.name,
+                "marking_2": self.ends[1].marking.name,
+                "lighting_1": self.ends[0].lighting.name,
+                "lighting_2": self.ends[1].lighting.name,
+                "tdz_lighting_1": self.ends[0].tdz_lighting,
+                "tdz_lighting_2": self.ends[1].tdz_lighting,
+                "reil_1": self.ends[0].reil.name,
+                "reil_2": self.ends[1].reil.name,
             },
         }
 
